%* lsaf_downloadaszip                                                                             *;
%*                                                                                                *;
%* Downloads SAS Life Science Analytics Framework files or a folder along with its content        *;
%* to your local machine as a zip file.                                                           *;
%*                                                                                                *;
%* If the local directory does not exist it will be created.  If the local file does not exist    *;
%* it will be created.  If the item being downloaded is a folder, the contents of the folder      *;
%* will be downloaded with its hierarchy maintained. If the item being downloaded is a context,   *;
%* only the containers (child contexts, folders) nd, if present, any files will be downloaded.    *;
%*                                                                                                *;
%* NOTE: This macro is only supported from a PC SAS session. Using this macro within an active    *;
%* SAS Life Science Analytics Framework application session will result in a failure.             *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_DOWNLOADASZIP(LSAF_PATH=lsaf-path, LOCAL_PATH=local-zipfile-path                         *;
%* <,LSAF_OVERWRITE=lsaf-overwrite> );                                                            *;
%*                                                                                                *;
%* @macvar _lsafRC_  the return code.                                                             *;
%* @macvar _lsafMsg_ the return message.                                                          *;
%*                                                                                                *;
%* @param lsaf_path - required - the path of the item (file or container) to be downloaded.       *;
%* @param local_path - required - the full path and name of the file to create (or overwrite,     *;
%*            if applicable) on the local computer. File extensions are entered on the file       *;
%*            as entered by the user. If no extension is entered it will result in zip file       *;
%*            with no extension.                                                                  *;
%* @param lsaf_overwrite - optional -  indicates whether or not to overwrite the existing zip     *;
%*            file on the local machine.  The valid values are 0 and 1 to maintain or overwrite   *;
%*            the file respectively.  The default is 0.                                           *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.4                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2013-08-30  updates for standardization and coding conventions                        *;
%* @history 2016-08-26  rebrand as LSAF                                                           *;
%* @history 2018-01-09  updates for LSAF 5.1                                                      *;
%*                                                                                                *;

%macro lsaf_downloadaszip(
  lsaf_path=,
  local_path=,
  lsaf_overwrite=0
  ) /des='Downloads a SAS Life Science Analytics Framework repository item as a zip file.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8
           message         length=$200
           logMsgPrefix    length=$70
           logMsgPrimary   length=$70
           logMsg          length=$300
    ;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/BatchRepositoryService") ;
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("downloadAsZip", "%bquote(&lsaf_path)", "%bquote(&local_path)", "%bquote(&lsaf_overwrite)", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Print messages in log           *;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not attempt the operation to download the item as a zip file: &lsaf_path.." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Attempted to download the item as a zip file: &lsaf_path.." ;
    end;
    
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;
    
    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_downloadaszip;