%* lsaf_getcheckedoutfiles                                                                         *;
%*                                                                                                 *;
%* Returns a SAS data set containing the checked out file information for the specified user,      *;
%*  file, and/or location within the SAS Life Science Analytics Framework repository. This action  *;
%*  requires the logged on user to have the administrative privilege                               *;
%*  PRIVILEGE_MANAGE_CHECKEDOUT_FILES. The SAS Life Science Analytics Framework limits the number  *; 
%*  of items that can be returned from a query. The macro variable _lsafCOFSearchLimitReached_     *;
%*  returns a 0 if this maximum limit is not reached and a 1 if the limit is reached. If the limit *;
%*  is reached there may still be checked out files in the repository that meet the given search   *;
%*  criteria. A more refined search will be needed to retrieve those records.                      *; 
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETCHECKEDOUTFILES(<LSAF_CHECKEDOUTBY=lsaf-checkedoutby,                                  *;
%*                           LSAF_NAME=lsaf-name,                                                  *;
%*                           LSAF_LOCATION=lsaf-location,                                          *;
%*                           SAS_DSNAME=SAS-data-set,                                              *;
%*                           LSAF_PRINTLIMITREACHEDWARNING = lsaf-print-limit-reached-warning>);   *;
%*                                                                                                 *;
%* @macvar _lsafRC_  the return code.                                                              *;
%* @macvar _lsafMsg_ the return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_checkedoutby - optional - The id of the user who checked out the files to be        *;
%*            retrieved. If set to * (ie lsaf_checkedoutby=*) or omitted, the files checked out    *;
%*            by any user will be returned. In addition wildcard notation can be used, for example *;
%*            lsaf_checkedoutby=ge* will return checked out by users starting with ge.             *;
%* @param lsaf_name - optional - The name of the file for which to retrieve checked out informa-   *;
%*            tion. If set to * (ie lsaf_name=*) or omitted, files of any name will be returned.   *;
%*            In addition wildcard notation can be used, for example lsaf_name=*.sas will return   *;
%*            all file names with a .sas extension.                                                *;
%* @param lsaf_location - optional - The folder path of the files for which to retrieve checked    *;
%*            out information. If set to * (ie lsaf_location=*) or omitted, files of from any      *;
%*            location will be returned. In addition wildcard notation can be used, for example    *;
%*            lsaf_location=/SAS/File* will return all information with a file path starting with  *;
%*            /SAS/File.                                                                           *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the        *;
%*            checked out files information for the specified checked out by user, name, and       *;
%*            location as libref.dataset. The default value is WORK.LSAFGETCHECKEDOUTFILES.        *;
%*            The output data set contains a single row for each checked out file and contains the *;
%*            following columns:                                                                   *;
%*                                                                                                 *;
%*            name            The name of the file                                                 *;
%*            size            The size of the file in bytes                                        *;
%*            formattedsize   The size of the file expressed in units such as KB, MB, etc.         *;
%*            checkedoutby    The user that has the file checked out                               *;
%*            modifieddate    The last modification date of the file                               *;
%*            version         The current version of the file that has been checked out            *;
%*            status          The workspace status of the file - NOT_CHECKED_OUT, CHECKED_OUT,     *;
%*                            and MARKED_FOR_ADD                                                   *;
%* @param lsaf_printLimitReachedWarning - optional - Determines whether or not to print the        *;
%*            message about the server query limit being reached as a WARNING or a NOTE. The       *;
%*            values are 0 to print message as a NOTE and 1 to print the message as a WARNING.     *;
%*            The default value is 1. This option will only apply when the                         *;   
%*            _lsafCOFSearchLimitReached_ macro is 1.                                              *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.5                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2016-12-16  initial coding.                                                            *;
%*                                                                                                 *;

%macro lsaf_getcheckedoutfiles(
   lsaf_checkedoutby=%str(_LSAF_VALUE_NOT_PROVIDED_),
   lsaf_name=%str(_LSAF_VALUE_NOT_PROVIDED_),
   lsaf_location=%str(_LSAF_VALUE_NOT_PROVIDED_),
   sas_dsname=work.lsafGetCheckedOutFiles,
   lsaf_printLimitReachedWarning=1
   ) /des='Gets a list of the checked out files.';

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
   %lsaf_getProductName();

   %global _lsafMsg_
           _lsafRC_
           _lsaf_printmessage_
   ;

   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

   %**********************************************;
   %* Start the datastep to call the java code   *;
   %**********************************************;
   data &sas_dsname (label="Created by &SYSMACRONAME"
                     keep=name location size formattedsize checkedoutby modifieddate version status
   );

   attrib
          rowCount        length=8
          returnCode      length=8
          message         length=$200
          logMsgPrefix    length=$70
          logMsg          length=$300
          logMsgPrimary   length=$200
          limitReached    length=8

          name            length=$256   label="File name"
          location        length=$2048  label="File location within LSAF"
          size            length=8      label="File size"
          formattedsize   length=$32    label="Formatted file size"
          checkedoutby    length=$64    label="Checked out by"
          modifieddate    length=$64    label="Date last modified"
          version         length=$32    label="File version"
          status          length=$32    label="File workspace status"
   ;

   rowCount=0;

   %***********************************;
   %* Declare the java objects        *;
   %***********************************;
   declare javaobj srv("com/sas/lsaf/macro/service/RepositoryService") ;
   declare javaobj results("com/sas/lsaf/macro/results/SasCheckedOutFilesResult");

   %***********************************;
   %* Attempt the operation           *;
   %***********************************;
   srv.callStaticVoidMethod("getCheckedOutFiles", "%bquote(&lsaf_checkedoutby)", "%bquote(&lsaf_name)", "%bquote(&lsaf_location)", results) ;

   %***********************************;
   %* Retrieve the results            *;
   %***********************************;
   results.callIntMethod("getReturnCode", returnCode);
   results.callStringMethod("getReturnMessage", message);
   results.callBooleanMethod("isLimitReached", limitReached);

   %***********************************;
   %* Process the results             *;
   %***********************************;
   logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
   if (returnCode ne 0) then
   do;
     logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
     logMsg=" Cannot retrieve checked out files.";
   end;
   else
   do;
     results.callIntMethod("getSize", rowCount);
     if limitReached then
     do;
       %lsaf_printwarningmessage(lsaf_printmessage=&lsaf_printlimitreachedwarning);
       logMsgPrimary="&_lsaf_PrintMessage_";
       logMsg=" The list of checked out files have been returned.";
     end;
     else 
     do;
       logMsgPrimary=logMsgPrefix;
       if (rowCount=0) 
         then logMsg = " No checked out files information was found.";
         else logMsg = " Checked out files information written to data set &sas_dsname";
     end;

     do row=0 to rowCount - 1;
       results.callStringMethod("getName", row, name);
       results.callStringMethod("getLocation", row, location);
       results.callLongMethod("getFileSize", row, size);
       results.callStringMethod("getFormattedSize", row, formattedsize);
       results.callStringMethod("getCheckedOutBy", row, checkedoutby);
       results.callStringMethod("getModifiedDate", row, modifieddate);
       results.callStringMethod("getVersion", row, version);
       results.callStringMethod("getWsStatus", row, status);
       output;
     end;
   end;

   %***********************************;
   %* Print messages in log           *;
   %***********************************;
   put;
   put logMsgPrimary " " logMsg;
   put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
   put logMsgPrefix " _lsafMsg_= " message ;
   put logMsgPrefix " _lsafRC_= " returnCode ;
   put logMsgPrefix " _lsafCOFSearchLimitReached_= " limitReached;
   put;

   %***********************************;
   %* Set the macro variables         *;
   %***********************************;
   call symputx("_lsafRC_", returnCode);
   call symputx("_lsafMsg_", message);
   call symputx("_lsafRowCount_", rowCount);
   call symputx("_lsafCOFSearchLimitReached_", limitReached);

   run;

%mend lsaf_getcheckedoutfiles;
