%* lsaf_getchildren                                                                              *;
%*                                                                                               *;
%* Returns a SAS data set that contains the metadata for items within a container in the SAS     *;
%* Life Science Analytics Framework repository.                                                  *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_GETCHILDREN(LSAF_PATH=lsaf-path <, LSAF_RECURSIVE=recursion-level,                      *;
%* SAS_DSNAME=SAS-data-set>);                                                                    *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_path - required - the path of the container item in the SAS Life Science          *;
%*            Analytics Framework repository.                                                    *;
%*                                                                                               *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the      *;
%*            metadata for the items specified as libref.dataset. The default value is           *;
%*            WORK.LSAFGETCHILDREN. The data set contains a row for each item in the list and    *;
%*            columns with the following names.  The data set will be sorted by path.            *;
%*            - name: The name of the item.                                                      *;
%*            - path: The path of the item.                                                      *;
%*            - itemType: Describes the type of the item (e.g. folder, data set, etc.)           *;
%*            - description: The description of the item.                                        *;
%*            - isContainer: A numeric flag that indicates whether the item is a container       *;
%*                  item (0=no, 1=yes).                                                          *;
%*            - isVersioned: A numeric flag that indicates whether the item is currently         *;
%*                  versioned (0=no, 1=yes).                                                     *;
%*            - version: The version associated with this specific item.                         *;
%*            - isCheckedOut: A  numeric flag that indicates whether the item is currently       *;
%*                  checked out (0=no, 1=yes).                                                   *;
%*            - isLocked: A numeric flag that indicates whether the item is currently locked     *;
%*                  (0=no, 1=yes).                                                               *;
%*            - size: The size of the item in bytes (will be 0 for containers).                  *;
%*            - createdBy: The user ID that created the item.                                    *;
%*            - created: The full date and time when the item was created. This value is         *;
%*                  represented as a String.                                                     *;
%*            - dateCreated: The date when the item was created as a SAS Datetime format.        *;
%*            - lastModifiedBy: The user ID that last modified the item.                         *;
%*            - lastModified: The date when the item was last modified.  This value is           *;
%*                  represented as a String.                                                     *;
%*            - dateLastModified: The date when the item was last modified as a SAS Datetime     *;
%*                  format.                                                                      *;
%*            - propertiesLastModifiedBy: The user ID that last modified the properties for the  *;
%*                  item.                                                                        *;
%*            - propertiesLastModified: The date when the item properties were last modified.    *;
%*                  This value is represented as a String.                                       *;
%*            - datePropertiesLastModified: The date when the item properties were last          *;
%*                  modified as a SAS Datetime format.                                           *;
%*            - state: This variable is for a context item only.  It indicates if the state      *;
%*                  state is ACTIVE or CLOSED.                                                   *;
%*            - stateComment: This variable is for contexts only and provides any                *;
%*                  comments attached by a user when changing the state of the context.          *;
%*            - signatureStatus: Indicates if the item has electronic signatures attached to it. *;
%*                  The following values can be returned:                                        *;
%*	            NONE = There are no signatures associated with this file.                    *;
%*	            PREVIOUS = There is at least one previous version of this file that is       *;
%*                      signed and the current version is not signed.                            *;
%*	            CURRENT = The current version of this file is signed, no previous versions   *;
%*                      are signed.                                                              *;
%*	            CURRENT_AND_PREVIOUS = The current version of this file is signed and there  *;
%*                      is at least one previous version of the file that is signed.             *;
%*                                                                                               *;
%* @param lsaf_recursive - optional - the level of recursion when the path is a container.       *;
%*            Valid values are 0, 1, 2, or 99 as described below. The default value is 0.  For   *;
%*            files, the value will be ignored.                                                  *;
%*                                                                                               *;
%*             0 - Returns the container only.                                                   *;
%*             1 - Returns the container and its immediate children.                             *;
%*             2 - Returns the container, its immediate children, and all items within one       *;
%*                 level of the containers subfolders.                                           *;
%*            99 - Returns the container and all of the sub containers and items                 *;
%*                 (traverses the entire tree).                                                  *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.1                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2013-06-28  version 1.2 update - path length                                         *;
%* @history 2013-08-30  updates for standardization and coding conventions                       *;
%* @history 2014-05-07  updated documentation and fixed data set processing.                     *;
%* @history 2016-08-29  rebrand as LSAF                                                          *;
%*                                                                                               *;

%macro lsaf_getchildren(
  lsaf_path=,
  sas_dsname=work.lsafGetChildren,
  lsaf_recursive=0
  ) /des='Get a list of the items in a folder in SAS Life Science Analytics Framework.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=path name description itemType isContainer
                         description version isVersioned isCheckedOut isLocked signatureStatus size
                         createdBy created dateCreated
                         lastModifiedBy lastModified dateLastModified
                         propertiesLastModifiedBy propertiesLastModified datePropertiesLastModified
                         state signatureStatus uppath);

    attrib returnCode                length=8
           message                   length=$200
           logMsgPrefix              length=$70
           logMsgPrimary             length=$70
           logMsg                    length=$300
           rowCount                  length=8

           path                      length=$2048    label="Item Path"
           name                      length=$256     label="Item Name"
           itemType                  length=$32      label="Item Type"
           isContainer               length=8        label="Is Item A Container?"

           description               length=$512     label="Description"
           version                   length=$32      label="Version Number"
           isVersioned               length=8        label="Is File Versioned"
           isCheckedOut              length=8        label="Is File Checked out"
           isLocked                  length=8        label="Is File Locked"
           signatureStatus           length=$32      label="Is File Signed"
           size                      length=8        label="Size In Bytes"

           createdBy                 length=$64      label="Userid that created the item"
           created                   length=$64      label="Item Creation Date"
           dateCreated                               label="Date the item was created (SAS DateTime Format)"

           lastModifiedBy            length=$64      label="Userid that last modified the item content"
           lastModified              length=$64      label="Last Content Modification Date"
           dateLastModified                          label="Date the item's content was last modified (SAS DateTime Format)"

           propertiesLastModifiedBy  length=$64      label="Userid that last modified the item properties"
           propertiesLastModified    length=$64      label="Last Properties Modification Date"
           datePropertiesLastModified                     label="Date the item's properties were last modified (SAS DateTime Format)"

           state                     length=$64      label="State of an Analysis or Project"

           createdTempSAS            length=$64      label="TEMPORARY - WILL BE DROPPED"
           lastModifiedTempSAS       length=$64      label="TEMPORARY - WILL BE DROPPED"
           uppath                    length=$2048    label="TEMPORARY - WILL BE DROPPED"
           propertiesLastModifiedTempSAS  length=$64 label="TEMPORARY - WILL BE DROPPED"
    ;

    format dateCreated
           dateLastModified
           datePropertiesLastModified datetime.
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java items        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/RepositoryService") ;
    declare javaobj results("com/sas/lsaf/macro/results/SasRepositoryItemsResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getChildren", "%bquote(&lsaf_path)", "%bquote(&lsaf_recursive)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not retrieve list of items from %bquote(&lsaf_path)" ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Returned a list of items in %bquote(&lsaf_path)";
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getPath", row, path);
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getDescription", row, description);
        results.callStringMethod("getVersion", row, version);
        results.callIntMethod("isContainer", row, isContainer);
        results.callIntMethod("isVersioned", row, isVersioned);
        results.callIntMethod("isCheckedOut", row, isCheckedOut);
        results.callIntMethod("isLocked", row, isLocked);
        results.callStringMethod("getSignatureStatus", row, signatureStatus);
        results.callLongMethod("getSize", row, size);
        results.callStringMethod("getItemType", row, itemType);
        results.callStringMethod("getCreated", row, created);
        results.callStringMethod("getCreatedBy", row, createdBy);
        results.callStringMethod("getLastModified", row, lastModified);
        results.callStringMethod("getLastModifiedBy", row, lastModifiedBy);
        results.callStringMethod("getPropertiesLastModified", row, propertiesLastModified);
        results.callStringMethod("getPropertiesLastModifiedBy", row, propertiesLastModifiedBy);
        results.callStringMethod("getState", row, state);

        results.callStringMethod("getCreatedSasFormat", row, createdTempSas);
        results.callStringMethod("getLastModifiedSasFormat", row, lastModifiedTempSas);
        results.callStringMethod("getPropertiesLastModifiedSasFormat", row, propertiesLastModifiedTempSAS);

        dateCreated=input(createdTempSAS, datetime.);
        dateLastModified=input(lastModifiedTempSas, datetime.);
        datePropertiesLastModified=input(propertiesLastModifiedTempSAS, datetime.);

        uppath=upcase(path) ;
        uppath=translate(uppath,'1','_');

        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No children were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " lsaf_recursive= &lsaf_recursive" ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    proc sort data=&sas_dsname;
      by uppath;
    run;
  %end;
  %***********************************;
  %* Cleanup                         *;
  %***********************************;
  proc sql;
     alter table &sas_dsname
        drop uppath;
  quit;

  options &notesOption;

%mend lsaf_getchildren;