%* lsaf_getcontexttypes                                                                            *;                                                       
%*                                                                                                 *;                                                 
%* Returns a SAS data set that contains the metadata for all of the context types defined in the   *;
%* SAS Life Science Analytics Framework repository                                                 *;
%*                                                                                                 *;                                                 
%* Syntax:                                                                                         *;                                                 
%* %LSAF_GETCONTEXTTYPES(SAS_DSNAME=SAS-data-set);                                                 *;                                                       
%*                                                                                                 *;                                                 
%* @macvar _lsafRC_  the return code.                                                              *;                                                
%* @macvar _lsafMsg_ the return message.                                                           *;                                                
%*                                                                                                 *;                                                 
%* @param sas_dsname - optional - the name of the output SAS data set that contains the metadata   *;                                                 
%*            for the context types specified as libref.dataset.  The default value is             *;                                                 
%*            WORK.LSAFGETCONTEXTTYPES.  The data set contains a row for each context, sorted by   *;                                                
%*            context ID, and columns with the following names:                                    *;                                                 
%*            - Id: The unique identifier for this context.                                        *;                                                 
%*            - Name: The display name for the context.                                            *;                                                 
%*            - Description: A description for the context.                                        *; 
%*            - LsafType: The LSAF type, in this case Context                                      *;
%*            - AllowableChildren: The ids for the immediate children of this context.             *;
%*            - Capabilities: The functionality allowed by the context.                            *;
%*                                                                                                 *;                                                 
%* @macrotype LSAF                                                                                 *;                                                
%* @since 2.1                                                                                      *;                                                 
%* @exposure external                                                                              *;                                                 
%*                                                                                                 *;                                                 

%macro lsaf_getcontexttypes(                                                                                                                              
  sas_dsname=work.lsafGetContextTypes                                                                                                                     
  ) /des='Get a list of all context types defined in the system.';                                                                                            
                                                                                                                                                      
  %****************************************;                                                                                                          
  %* Initialize macro variables           *;                                                                                                          
  %****************************************;                                                                                                          
  %lsaf_getProductName();

  %global _lsafMsg_                                                                                                                                   
          _lsafRC_                                                                                                                                    
  ;                                                                                                                                                   
                                                                                                                                                      
  %local _lsafRowCount_;                                                                                                                              
                                                                                                                                                      
  %let _lsafRC_ =%str(-999);                                                                                                                          
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);                                                                 
  %let _lsafRowCount_=0;                                                                                                                              
                                                                                                                                                      
  %**********************************************;                                                                                                    
  %* Start the datastep to call the java code   *;                                                                                                    
  %**********************************************;                                                                                                    
  data &sas_dsname (label="Created by &SYSMACRONAME"                                                                                                  
                    keep= id name description lsaftype allowablechildren capabilities typeid);                                                                                                                                                  
                                                                                                                                                      
    attrib returnCode              length=8;                                                                                                          
    attrib message                 length=$200;                                                                                                       
    attrib logMsgPrefix            length=$70;                                                                                                        
    attrib logMsgPrimary           length=$70;                                                                                                        
    attrib logMsg                  length=$300;                                                                                                       
    attrib rowCount                length=8;                                                                                                          
                                                                                                                                                      
    attrib id                      length=$64    label="Context ID";                                                                                     
    attrib TYPEID                  length=$64    label="NEEDED FOR SORTING - will be dropped";                                                        
    attrib name                    length=$128   label="Context Name";                                                                                
    attrib description             length=$1024  label="Context Description";                                                                                  
    attrib lsafType                length=$64    label="LSAF Context Type";                                                                                  
    attrib allowableChildren       length=$128   label="Allowable Children Type";                                                                                   
    attrib capabilities            length=$128   label="Capabilities";                                                                               
                                                                                                                                                      
    rowCount=0;                                                                                                                                       
                                                                                                                                                      
    %***********************************;                                                                                                             
    %* Declare the java objects        *;                                                                                                             
    %***********************************;                                                                                                             
    declare javaobj srv("com/sas/lsaf/macro/service/TypeService") ;                                                                                    
    declare javaobj results("com/sas/lsaf/macro/results/SasTypesResult");                                                                              
                                                                                                                                                      
    %***********************************;                                                                                                             
    %* Attempt the operation           *:                                                                                                             
    %***********************************;                                                                                                             
    srv.callStaticVoidMethod("getContextTypes", results);                                                                                                 
                                                                                                                                                      
    %***********************************;                                                                                                             
    %* Retrieve the results            *;                                                                                                             
    %***********************************;                                                                                                             
    results.callIntMethod("getReturnCode", returnCode);                                                                                               
    results.callStringMethod("getReturnMessage", message);                                                                                            
                                                                                                                                                      
    %***********************************;                                                                                                             
    %* Process the results             *;                                                                                                             
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then                                                                                                                         
    do;                                                                                                                                               
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";                                                                                        
      logMsg=" Cannot retrieve list of context types.";                                                                                                       
    end;                                                                                                                                              
    else                                                                                                                                              
    do;                                                                                                                                               
      logMsgPrimary=logMsgPrefix;                                                                                            
      logMsg=" List of context types was returned.";                                                                                                          
      results.callIntMethod("getSize", rowCount);                                                                                                     
      do row=0 to rowCount - 1;                                                                                                                       
        results.callStringMethod("getId", row, id);                                                             
        results.callStringMethod("getName", row, name);                                                             
        results.callStringMethod("getDescription", row, description);                                                         
        results.callStringMethod("getLsafType", row, lsafType);                                                                         
        results.callStringMethod("getAllowableChildTypes", row, allowableChildren);                                                                     
        results.callStringMethod("getCapabilities", row, capabilities);                                                                         
                                                                                                                                                      
        TYPEID=upcase(Id);                                                                                                                      
                                                                                                                                                      
        output;                                                                                                                                       
      end;                                                                                                                                            
    end;                                                                                                                                              
                                                                                                                                                      
    %***********************************;                                                                                                             
    %* Print messages in log           *;                                                                                                             
    %***********************************;                                                                                                             
    put;                                                                                                                                              
    put logMsgPrimary " " logMsg;                                                                                                                      
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));                                                                                          
    if (rowCount=0) 
      then put logMsgPrefix " No context types were found.";                                                                                                       
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";                                                                       
    put logMsgPrefix " _lsafMsg_= " message ;                                                                                                         
    put logMsgPrefix " _lsafRC_= " returnCode ;                                                                                                       
    put;                                                                                                                                              
                                                                                                                                                      
    %***********************************;                                                                                                             
    %* Set the macro variables         *;                                                                                                             
    %***********************************;                                                                                                             
    call symputx("_lsafRC_", returnCode);                                                                                                             
    call symputx("_lsafMsg_", message);                                                                                                               
    call symputx("_lsafRowCount_", rowCount);                                                                                                         
                                                                                                                                                      
  run;                                                                                                                                                
                                                                                                                                                      
  %let notesOption=%sysfunc(getoption(notes));                                                                                                        
  options nonotes;                                                                                                                                    
  %***********************************;                                                                                                               
  %* Sort the data set               *;                                                                                                               
  %***********************************;                                                                                                               
  %if (&_lsafRowCount_ > 0) %then                                                                                                                     
  %do;                                                                                                                                                
    proc sort data=&sas_dsname;                                                                                                                       
      by typeid;                                                                                                                                    
    run;                                                                                                                                              
  %end;                                                                                                                                               
  %***********************************;                                                                                                               
  %* Cleanup                         *;                                                                                                               
  %***********************************;                                                                                                               
  proc sql;                                                                                                                                           
     alter table &sas_dsname                                                                                                                          
        drop typeid;                                                                                                                                
  quit;                                                                                                                                               
                                                                                                                                                      
  options &notesOption;                                                                                                                               
                                                                                                                                                      
%mend lsaf_getcontexttypes;