%* lsaf_updatecontextstate                                                                         *;                                                                                                                                                                                                                                
%*                                                                                                 *;                                                                                                                                                                                                                                
%* Updates the state of a context in the SAS Life Science Analytics Framework repository.          *;
%* Attempting to set the state of context that is already in that state will just ignore the       *;
%* action with no error. The lsaf_recursive parameter only applies when reactivating a context.    *;
%* The behavior for closing a context is to automatically close all children.                      *;
%*                                                                                                 *;
%* To determine if a context can have its state altered, the capabilities column from the output   *;
%* data set generated by %lsaf_getcontexttypes will contain the value STATE.                       *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;                                                                                                                                                                                                                                
%* %LSAF_UPDATECONTEXTSTATE(LSAF_PATH=lsaf-path, LSAF_STATE=lsaf-state,                            *;                                                                                                                                                                                                                                
%* < LSAF_COMMENT=lsaf-comment, LSAF_RECURSIVE=lsaf-recursive>);                                   *;                                                                                                                                                                                                                                
%*                                                                                                 *;                                                                                                                                                                                                                                
%* @macvar _lsafRC_  the return code.                                                              *;                                                                                                                                                                                                                                
%* @macvar _lsafMsg_ the return message.                                                           *;                                                                                                                                                                                                                                
%*                                                                                                 *;                                                                                                                                                                                                                                
%* @param lsaf_path - required - the path of the context to be updated.                            *;                                                                                                                                                                                                                                
%* @param lsaf_state - required - the state to set on the context.                                 *;                                                                                                                                                                                                                                
%*            Values:  ACTIVE | CLOSED                                                             *;                                                                                                                                                                                                                                
%* @param lsaf_comment - optional - the reason for the state change.                               *;                                                                                                                                                                                                                                
%* @param lsaf_recursive - optional - whether context children, if any exist, should have their    *;                                                                                                                                                                                                                                
%*            state updated as well. The recursive flag only applies when reactivating a context.  *;
%*            The behavior for closing a context is to automatically close all children.           *;                                                                                                                                                                                                                                
%*            Values: 0 | 1                                                                        *;                                                                                                                                                                                                                                
%*            If no value is supplied the default value of 0 is applied.                           *; 
%*                                                                                                 *;                                                                                                                                                                                                                                
%* @macrotype LSAF                                                                                 *;                                                                                                                                                                                                                                
%* @since 2.1                                                                                      *;                                                                                                                                                                                                                                
%* @exposure external                                                                              *;                                                                                                                                                                                                                                
%*                                                                                                 *;                                                                                                                                                                                                                                
%* @history 2019-01-29  initial coding                                                             *;
%*                                                                                                 *;                                                                                                                                                                                                                                
                                                                                                                                                                                                                                                                                                                                     
%macro lsaf_updatecontextstate(                                                                                                                                                                                                                                                                                                      
  lsaf_path=,                                                                                                                                                                                                                                                                                                                        
  lsaf_state=,                                                                                                                                                                                                                                                                                                                       
  lsaf_comment=,                                                                                                                                                                                                                                                                                                                     
  lsaf_recursive=0                                                                                                                                                                                                                                                                                                            
  ) /des='Update the state of a context.';                                                                                                                                                                                                                                                                                           
                                                                                                                                                                                                                                                                                                                                     
  %****************************************;                                                                                                                                                                                                                                                                                         
  %* Initialize macro variables           *;                                                                                                                                                                                                                                                                                         
  %****************************************;                                                                                                                                                                                                                                                                                         
  %lsaf_getProductName();                                                                                                                                                                                                                                                                                                            
                                                                                                                                                                                                                                                                                                                                     
  %global _lsafMsg_                                                                                                                                                                                                                                                                                                                  
          _lsafRC_                                                                                                                                                                                                                                                                                                                   
  ;                                                                                                                                                                                                                                                                                                                                  
                                                                                                                                                                                                                                                                                                                                     
  %let _lsafRC_=%str(-999);                                                                                                                                                                                                                                                                                                          
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);                                                                                                                                                                                                                                                
                                                                                                                                                                                                                                                                                                                                     
  %**********************************************;                                                                                                                                                                                                                                                                                   
  %* Start the datastep to call the java code   *;                                                                                                                                                                                                                                                                                   
  %**********************************************;                                                                                                                                                                                                                                                                                   
  data _null_;                                                                                                                                                                                                                                                                                                                       
    attrib returnCode      length=8;                                                                                                                                                                                                                                                                                                 
    attrib message         length=$200;                                                                                                                                                                                                                                                                                              
    attrib logMsgPrefix    length=$70;                                                                                                                                                                                                                                                                                               
    attrib logMsgPrimary   length=$70;                                                                                                                                                                                                                                                                                               
    attrib logMsg          length=$300;                                                                                                                                                                                                                                                                                              
                                                                                                                                                                                                                                                                                                                                     
    %***********************************;                                                                                                                                                                                                                                                                                            
    %* Declare the java objects        *;                                                                                                                                                                                                                                                                                            
    %***********************************;                                                                                                                                                                                                                                                                                            
    declare javaobj srv("com/sas/lsaf/macro/service/RepositoryService");                                                                                                                                                                                                                                                                 
    declare javaobj results("com/sas/lsaf/macro/results/Result");                                                                                                                                                                                                                                                                     
                                                                                                                                                                                                                                                                                                                                     
    %***********************************;                                                                                                                                                                                                                                                                                            
    %* Attempt the operation           *:                                                                                                                                                                                                                                                                                            
    %***********************************;                                                                                                                                                                                                                                                                                            
    srv.callStaticVoidMethod("setContextState", "%bquote(&lsaf_path)", "%bquote(&lsaf_state)", "%bquote(&lsaf_comment)",                                                                                                                                                                                                                 
              "%bquote(&lsaf_recursive)", results);                                                                                                                                                                                                                                                                           
                                                                                                                                                                                                                                                                                                                                     
    %***********************************;                                                                                                                                                                                                                                                                                            
    %* Retrieve the results            *;                                                                                                                                                                                                                                                                                            
    %***********************************;                                                                                                                                                                                                                                                                                            
    results.callIntMethod("getReturnCode", returnCode);                                                                                                                                                                                                                                                                              
    results.callStringMethod("getReturnMessage", message);                                                                                                                                                                                                                                                                           
                                                                                                                                                                                                                                                                                                                                     
    %***********************************;                                                                                                                                                                                                                                                                                            
    %* Process the results             *;                                                                                                                                                                                                                                                                                            
    %***********************************;                                                                                                                                                                                                                                                                                            
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    logMsgPrimary=logMsgPrefix;
    logMsg=" The state for context &lsaf_path was updated to &lsaf_state.." ;                                                                                                                                                                                                                                                                                  
    if (returnCode ne 0) then                                                                                                                                                                                                                                                                                                        
    do;                                                                                                                                                                                                                                                                                                                              
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";                                                                                                                                                                                                                                                                       
      logMsg=" Cannot update context &lsaf_path." ;                                                                                                                                                                                                                                                                                  
    end;                                                                                                                                                                                                                                                                                                                             
                                                                                                                                                                                                                                                                                                                                     
    %***********************************;                                                                                                                                                                                                                                                                                            
    %* Print messages in log           *;                                                                                                                                                                                                                                                                                            
    %***********************************;                                                                                                                                                                                                                                                                                            
    put;                                                                                                                                                                                                                                                                                                                             
    put logMsgPrimary " " logMsg;                                                                                                                                                                                                                                                                                                     
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));                                                                                                                                                                                                                                                                         
    put logMsgPrefix " _lsafMsg_= " message ;                                                                                                                                                                                                                                                                                        
    put logMsgPrefix " _lsafRC_= " returnCode ;                                                                                                                                                                                                                                                                                      
    put;                                                                                                                                                                                                                                                                                                                             
                                                                                                                                                                                                                                                                                                                                     
    %***********************************;                                                                                                                                                                                                                                                                                            
    %* Set the macro variables         *;                                                                                                                                                                                                                                                                                            
    %***********************************;                                                                                                                                                                                                                                                                                            
    call symputx("_lsafRC_", returnCode);                                                                                                                                                                                                                                                                                            
    call symputx("_lsafMsg_", message);                                                                                                                                                                                                                                                                                              
                                                                                                                                                                                                                                                                                                                                     
  run;                                                                                                                                                                                                                                                                                                                               
                                                                                                                                                                                                                                                                                                                                     
%mend lsaf_updatecontextstate;                                                                                                                                                                                                                                                                                                       