%* lsaf_uploadandexpand                                                                           *;
%*                                                                                                *;
%* Uploads and expands the contents of a zip file into the specified folder location in the SAS   *;
%* Life Science Analytics Framework repository.                                                   *;
%*                                                                                                *;
%* NOTE: This macro is only supported from a PC SAS session. Using this macro within an active    *;
%* SAS Life Science Analytics Framework application session will result in a failure.             *;
%*                                                                                                *;
%* Failure and Message Handling:                                                                  *;
%* The action status for each file that is being uploaded and each folder that is being created   *;
%* will be printed in the log.                                                                    *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_UPLOADANDEXPAND(LOCAL_PATH=local-zipfile-path, LSAF_PATH=lsaf-path                       *;
%* <, LSAF_ENABLEVERSIONING=lsaf-enable-versioning, LSAF_VERSIONTYPE=lsaf-version-type,           *;
%*    LSAF_COMMENT=lsaf_comment>);                                                                *;
%*                                                                                                *;
%* @macvar _lsafRC_  the return code.                                                             *;
%* @macvar _lsafMsg_ the return message.                                                          *;
%*                                                                                                *;
%* @param local_path - required - the absolute path and name of the zip file on the local         *;
%*            computer.                                                                           *;
%* @param lsaf_path - required - the path of the target folder.                                   *;
%* @param lsaf_enableversioning - optional - indicates whether the new files being created should *;
%*            be versioned. The default value is 0, which will create a non-versioned file.  A    *;
%*            value of 1 will create a versioned file. The versioning status of existing          *;
%*            files will not be changed.                                                          *;
%*            Values:  0 | 1                                                                      *;
%*            Default: 0                                                                          *;
%* @param lsaf_versiontype - optional - the type of version to be created if the new files are to *;
%*            be versioned. Valid values are MAJOR or MINOR. The default is MAJOR. This value is  *;
%*            ignored for non-versioned items.                                                    *;
%* @param lsaf_comment - conditional - the comment for the versioned file being created. This     *;
%*            option is ignored when creating non-versioned files.                                *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.4                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2013-03-18  initial code                                                              *;
%* @history 2016-08-26  rebrand as LSAF                                                           *;
%* @history 2018-01-09  updates to for LSAF 5.1                                                   *;
%*                                                                                                *;

%macro lsaf_uploadandexpand(
  local_path=,
  lsaf_path=,
  lsaf_enableversioning=,
  lsaf_versiontype=,
  lsaf_comment=
  ) /des='Uploads and expands a zip file into the SAS Life Science Analytics Framework repository.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8
           message         length=$200
           logMsgPrefix    length=$70
           logMsgPrimary   length=$70
           logMsg          length=$300
           rowCount        length=8

           sasLogFlag      length=$200
           sasLogAction    length=$200
           detailPath      length=$2048
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/BatchRepositoryService") ;
    declare javaobj results("com/sas/lsaf/macro/results/SasActionStatusResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("uploadAndExpand", "%bquote(&local_path)", "%bquote(&lsaf_path)", "%bquote(&lsaf_enableversioning)",
         "%bquote(&lsaf_versiontype)", "%bquote(&lsaf_comment)", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);
    results.callIntMethod("getSize", rowCount);

    %***********************************;
    %* Print messages in log           *;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not attempt the operation to upload and expand the file to target path: &lsaf_path.." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Attempted the operation to upload and expand the file to target path: &lsaf_path.." ;
    end;

    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_ = " returnCode ;


    if (rowCount > 0) then
    do;
       macroNameStamp=strip(lowcase("&SYSMACRONAME")) || ": ";
       do row=0 to rowCount - 1;
         results.callStringMethod("getDetailPath", row, detailPath);
         results.callStringMethod("getDetailStatus", row, sasLogFlag);
         results.callStringMethod("getDetailMessage", row, sasLogAction);
         sasLogFlag   = strip(sasLogFlag)   || ": ";
         sasLogAction = strip(sasLogAction) || ": ";
         put sasLogFlag macroNameStamp sasLogAction detailPath;
       end;
    end;
    put "-" ;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_uploadandexpand;
